`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2019/04/22 19:35:26
// Design Name: 
// Module Name: speaker_control
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

module speaker_control(
    clk,
    rst,
    audio_in_left,
    audio_in_right,
    audio_mclk,
    audio_lrck,
    audio_sck,
    audio_sdin,
    key_release
    );
    
    output audio_mclk, audio_lrck, audio_sck;
    output reg audio_sdin;
    input clk, rst;
    input [15:0] audio_in_left, audio_in_right;
    input key_release;

    reg [15:0] audio_tmp;
    reg [3:0] audio_bck_counter;
    reg [3:0] audio_bck_counter_tmp;

    wire audio_left_right_sel;

    freqdiv_25M U0(
    .mclk(audio_mclk), //divided clock output
    .clk(clk),    //clock input
    .rst(rst)   //low-active reset
);
    freqdiv_25M_divide_128 U1(
    .lrck(audio_lrck), //divided clock output
    .clk(clk),    //clock input
    .rst(rst)   //low-active reset
);
    freqdiv_25M_divide_4 U2(
    .sck(audio_sck), //divided clock output
    .clk(clk),    //clock input
    .rst(rst)   //low-active reset
);
    
    assign audio_left_right_sel = audio_lrck;

        always @*
		    audio_bck_counter_tmp = audio_bck_counter + 1'b1;

	    always @(posedge audio_sck or posedge rst)
		    if (rst)
			    audio_bck_counter = 4'b0;
		    else
		        audio_bck_counter = audio_bck_counter_tmp;

	    always @* // if audio_left_right_sel = 0, then from left. Otherwise, from right.
	    if (~key_release)
            audio_sdin = 1'b0;
        else    
            case ({audio_left_right_sel, audio_bck_counter})
		    5'b00000:
                audio_sdin = audio_in_left[15];
		    5'b00001:
                audio_sdin = audio_in_left[14];
		    5'b00010:
                audio_sdin = audio_in_left[13];
		    5'b00011: 
                audio_sdin = audio_in_left[12];
		    5'b00100: 
                audio_sdin = audio_in_left[11];
		    5'b00101: 
                audio_sdin = audio_in_left[10];
		    5'b00110: 
                audio_sdin = audio_in_left[9];
		    5'b00111: 
                audio_sdin = audio_in_left[8];
		    5'b01000: 
                audio_sdin = audio_in_left[7];
		    5'b01001: 
                audio_sdin = audio_in_left[6];
		    5'b01010: 
                audio_sdin = audio_in_left[5];
		    5'b01011: 
                audio_sdin = audio_in_left[4];
		    5'b01100: 
                audio_sdin = audio_in_left[3];
		    5'b01101: 
                audio_sdin = audio_in_left[2];
		    5'b01110: 
                audio_sdin = audio_in_left[1];
		    5'b01111: 
                audio_sdin = audio_in_left[0];
		    5'b10000: 
                audio_sdin = audio_in_right[15];
		    5'b10001: 
                audio_sdin = audio_in_right[14];
		    5'b10010: 
                audio_sdin = audio_in_right[13];
		    5'b10011: 
                audio_sdin = audio_in_right[12];
		    5'b10100: 
                audio_sdin = audio_in_right[11];
		    5'b10101: 
                audio_sdin = audio_in_right[10];
	        5'b10110: 
                audio_sdin = audio_in_right[9];
		    5'b10111: 
                audio_sdin = audio_in_right[8];
		    5'b11000: 
                audio_sdin = audio_in_right[7];
		    5'b11001: 
                audio_sdin = audio_in_right[6];
		    5'b11010: 
                audio_sdin = audio_in_right[5];
		    5'b11011: 
                audio_sdin = audio_in_right[4];
		    5'b11100: 
                audio_sdin = audio_in_right[3];
		    5'b11101: 
                audio_sdin = audio_in_right[2];
		    5'b11110: 
                audio_sdin = audio_in_right[1];
		    5'b11111: 
                audio_sdin = audio_in_right[0];
            default: audio_sdin = 1'b0;
        endcase
endmodule




