`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2019/04/22 19:35:26
// Design Name: 
// Module Name: topmodule
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

module topmodule(
    D_ssd,
    ssd_ctl,
    clk, // clock from crystal
    rst, 
    audio_mclk, // master clock
    audio_lrck, // left-right clock
    audio_sck, // serial clock
    audio_sdin, // serial audio data input
    PS2_DATA,
    PS2_CLK,
    DIP_double_tone
    
);

    input clk; // clock from the crystal
    input rst; 
    input DIP_double_tone;
    output audio_mclk; // master clock
    output audio_lrck; // left-right clock
    output audio_sck; // serial clock
    output audio_sdin; // serial audio data input
    output [7:0] D_ssd;
    output [3:0] ssd_ctl;

    inout PS2_CLK;
    inout PS2_DATA;

    wire [511:0] key_down;
    wire [8:0] last_change;
    wire key_valid;

    wire clk_1Hz;
    wire clk_100Hz;
    wire [1:0] clk_ssd_ctl;
    
    wire [15:0] audio_in_left, audio_in_right;
    wire [21:0] note_result_left, note_result_right;

    wire [3:0] in_1, in_0;
    wire [3:0] ssd_in;

    wire [6:0] select_in;
    
    assign valid_signal = key_down[last_change];

    clock_generator U0(
    .clk_1Hz(clk_1Hz), //divided clock output
    .clk_100Hz(clk_100Hz),
    .clk_ctl(clk_ssd_ctl), //clock for 7-segment
    .clk(clk),    //clock input
    .rst(rst)   //low-active reset
);

    KeyboardDecoder U1(
	.key_down(key_down),
	.last_change(last_change),
	.key_valid(key_valid),
	.PS2_DATA(PS2_DATA),
	.PS2_CLK(PS2_CLK),
	.rst(rst),
	.clk(clk)
    );

    key_release U2(
    .key_valid(key_valid),
    .clk(clk_100Hz),
    .rst(rst),
    .curr_state(key_release)
    );

    fsm U3(
    .last_change(last_change),
    .valid(valid_signal),
    .curr_state(caps_lock),
    .rst(rst)
);

    ASCII_decoder U4(
    .ASCII_code(select_in), 
    .key_down(key_down),
    .last_change(last_change),
    .caps_lock(caps_lock),
    .rst(rst),
    .clk(clk),
    .key_release(key_release)
    );


    key_to_note_decoder U5(
    .note_result_left(note_result_left),
    .note_result_right(note_result_right), 
    .select_in(select_in),
    .clk(clk),
    .rst(rst),
    .display_in_1(in_1),
    .display_in_0(in_0),
    .DIP_double_tone(DIP_double_tone)
    );

       

    note_gen Ung(
    .clk(clk), // clock from crystal
    .rst(rst), // active low reset
    .note_div_left(note_result_left), // div for note generation
    .note_div_right(note_result_right),
    .audio_left(audio_in_left), // left sound audio
    .audio_right(audio_in_right) // right sound audio
    );

// Speaker controllor
    speaker_control Usc(
    .clk(clk), // clock from the crystal
    .rst(rst), // active low reset
    .audio_in_left(audio_in_left), // left channel audio data input
    .audio_in_right(audio_in_right), // right channel audio data input
    .audio_mclk(audio_mclk), // master clock
    .audio_lrck(audio_lrck), // left-right clock
    .audio_sck(audio_sck), // serial clock
    .audio_sdin(audio_sdin), // serial audio data input
    .key_release(key_release)
    );

    scan_freq_ctl U7(
    .ssd_in(ssd_in),
    .ssd_ctl(ssd_ctl),
    .clk_ctl_en(clk_ssd_ctl),
    .in_1(in_1),
    .in_0(in_0)
);

    display U8(
    .D_ssd(D_ssd), 
    .i(ssd_in)
    );
endmodule



